/*
 * Copyright (c) 2020 Pedro Quaresma
 *
 * Utilizaao: forcaBrutaDS nome_ficheiro_entrada ficheiro_saida 
 *               lista_de_palavras
 * 
 * -> ficheiro contendo o texto cifrado
 * -> lista de palavras "listaPalavras.txt", contendo a lista de 
 *       palavras Portuguesas pr-Acordo Ortogrfico
 * <- ficheiro contendo a chave e o texto decifrado
 *
 * Procede por tentativas, desde a chave nula at ao tamanho mximo do
 * alfabeto Usa a lista de palavras para testar as concordncias.  O
 * resultado final  dado pelo caso em que foram encontradas mais
 * concordncias
 */

#include <iostream>
#include <string>

#include <cstdio>
#include <cstring> // permite a utilizao de funes que manipulam strings 
#include <cstdlib> // biblioteca para funcao exit
#include <cctype>  // para ispace and ispunct
#include <climits>
#include <cstdint>

#include "cifrasSubstituicao.hpp" // Cifras de Substituio
#include "dicPT.hpp" // Dicionrio Portugus

using namespace std;

int main(int argc, char *argv[]){

  //verificar o nmero de argumento
  if(argc<3){
    cout << "\nUtilizaao: forcaBrutaDS nome_ficheiro_entrada ficheiro_saida\n\n";
    return(1);
  }

  // fl  - o ficheiro de entrada (leitura) 
  // fe  - o ficheiro de saida (escrita)
  FILE *fl,*fe;

  //Declarao das variveis
  char *menscifrada, *mensdecifrada;
  int ch;
  int resultado;
  int chave;

  char palavra[40]; // palavra com, no mximo 30 caracteres
  int compPalavra = 0;
  int numPalavras = 0; // nmero de palavras encontradas numa tentativa
  int maiorConcordancia = 0; // maior nmero de palavras encontradas;
  int chaveMaisProvavel = 0; // chave mais provvel (maior nmero de concordncia)
  
  // Objecto do tipo Cifras de Deslocamento Simples
  CifrasSubstituicao cifraDS;
  // Objecto do tipo FuncoesAuxiliares
  DicionarioPT dicionarioPT;

  //ficheiro de entrada (encriptado)
  menscifrada = argv[1];
   
  // ficheiros de sada (desencriptado)
  mensdecifrada = argv[2];
  
  // abrir o ficheiro para a leitura (interromper se erro)
  if((fl = fopen(menscifrada,"r")) == NULL){
    cout << "Impossivel abrir o ficheiro " << argv[1] << endl;
    return(2);
  }
  
  // abrir o ficheiro para escrever a mensagem decifrada
  if((fe = fopen(mensdecifrada,"w")) == NULL){
    cout << "Impossivel criar o ficheiro " << mensdecifrada << endl;
    return(3);
  }
  
  // Est tudo bem com os ficheiros, vai-se iniciar o programa
  chave = 0;  
  do {
    //Aplicar a Cifra de Deslocamento Simples
    while((ch=fgetc(fl)) != EOF){
      resultado = cifraDS.desencriptarDS(ch,chave);
      // vai construir a palavra, sequncia de caracters que no sejam
      // espaos ou sinais de pontuao
      if (!isspace(resultado) && !ispunct(resultado)) {
	palavra[compPalavra] = resultado;
	compPalavra++;
      }
      else {
	if (compPalavra != 0) {
	  palavra[compPalavra] = '\0';
	  // se a palavra pertence ao dicionrio Pt, incrementa-se o
	  // contador
	  if (dicionarioPT.pertence(palavra) == 1) {
	    numPalavras++;
	  }
	  compPalavra = 0;
	}
      }
    }
    // actualiza-se, se necessrio, o maior nmero de concordncias
    // encontradas, por chave
    if (numPalavras >= maiorConcordancia) {
      chaveMaisProvavel = chave;
      maiorConcordancia = numPalavras;
    }
    //    printf("\n%d --- %d\n", chave,numPalavras);
    chave++;
    numPalavras=0;
    // fecha o ficheiro da mensagem encriptada
    fclose(fl);
    // para o voltar a abrir, nova tentativa
    fl = fopen(menscifrada,"r");

  } while (chave <= cifraDS.compAlfa);

  //Fechar o ficheiro da mensagem encriptada
  fclose(fl);

  // para o voltar a abrir
  fl = fopen(menscifrada,"r");

  if (maiorConcordancia > 0) {
    fprintf(fe,"\n\nChave mais provvel: %d\n\n",chaveMaisProvavel);
    // Aplicar a Cifra de Deslocamento Simples
    while((ch=fgetc(fl))!=EOF){
      resultado = cifraDS.desencriptarDS(ch,chaveMaisProvavel);
      fprintf(fe,"%c",resultado);
    }
  }
  else {
    cout << "Chave no encontrada\n";
  }

  //Fechar os ficheiros
  fclose(fl);
  fclose(fe);
  
  exit(0);
}
